/*
 * Copyright (c) 2014 by ELECTRONIC ASSEMBLY <technik@lcd-module.de>
 * EA OLED Graphic (SSD1309) software library for arduino.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU General Public License version 2
 * or the GNU Lesser General Public License version 2.1, both as
 * published by the Free Software Foundation.
 */

#include <Arduino.h>
#include <Wire.h>
#include <avr/pgmspace.h>

#include "oled_SSD1309_IIC.h"


#define INITLEN 11
byte init_OLEDL128[INITLEN] = {0x40, 0xA0, 0xC0, 0xA6, 0x81, 0xFF, 0xD5, 0x40, 0xD9, 0x44, 0xAf};
byte init_OLEDM128[INITLEN] = {0x40, 0xA0, 0xC0, 0xA6, 0x81, 0xFF, 0xD5, 0x40, 0xD9, 0x44, 0xAf};

//----------------------------------------------------public Functions----------------------------------------------------
//Please use these functions in your sketch

/*----------------------------
Func: OLED-INIT
Desc: Initializes SPI Hardware/Software and OLED Displays
Vars: CS-Pin, MOSI-Pin, SCK-Pin (MOSI=SCK Hardware else Software), A0-Pin (high=data, low=command), p_res = Reset-Pin, type (1=EA OLEDM128-6, 2=EA OLEDL128-6)
------------------------------*/
void oled_SSD1309::initialize(byte p_res, byte address, byte type) 
{
	byte *ptr_init; //pointer to the correct init values
	top_view = false; //default = bottom view
	
	iic_address = address;
	Wire.begin();
	Wire.setClock(100000);
	
	Serial.begin(9600);  // start serial for output

	//perform a Reset
	digitalWrite(p_res, LOW);
	pinMode(p_res, OUTPUT);
	delayMicroseconds(10);
	digitalWrite(p_res, HIGH);
	delay(1);

	//Init OLED displays, depending on users choice
	ptr_init = init_OLEDL128; //default pointer for wrong parameters
	if(type == OLEDM128) 		ptr_init = init_OLEDM128;

	
	oled_SSD1309::type = type;

	iic_put(ptr_init, INITLEN, 1);

	clear();
}

/*----------------------------
Func: clear_display
Desc: clears the entire OLED-Display
Vars: ---
------------------------------*/
void oled_SSD1309::clear(void) 
{
	byte page, column;
	
	
	for(page = 0; page < 8; page++) //Display has 8 pages
	{
		position(0,page);
		
	
		for(column = 0; column < 128; column++) //clear the whole page line
			iic_put_byte(0,0);
		
	}
}

/*----------------------------
Func: contrast
Desc: sets contrast to the OLED-Display
Vars: byte contrast (0..63)
------------------------------*/
void oled_SSD1309::contrast(byte contr) 
{
	command(0x81);   		//double byte command
	command(contr&0x3F);	//contrast has only 6 bits
}

/*----------------------------
Func: view
Desc: ssets the display viewing direction
Vars: direction (top view 0xC8, bottom view (default) = 0xC0)
------------------------------*/
void oled_SSD1309::view(byte direction)  
{
	if(direction == VIEW_TOP)
	{
		top_view = true;
		command(0xA1);
		command(0xC8);
	}
	else
	{
		top_view = false;
		command(0xA0);
		command(0xC0);
	}
		
	//clear(); //Clear screen, as old content is not usable (mirrored)
}

/*----------------------------
Func: string
Desc: shows string with selected font on position
Vars: column (0..127/131), page(0..3/7),  font adress in programm memory, stringarray
------------------------------*/
void oled_SSD1309::string(byte column, byte page, const byte *font_adress, const char *str)
{
	unsigned int pos_array; 										//Postion of character data in memory array
	byte x, y, column_cnt, width_max;								//temporary column and page adress, couloumn_cnt tand width_max are used to stay inside display area
	byte start_code, last_code, width, page_height, bytes_p_char;	//font information, needed for calculation
	const char *string;

	
	
	start_code 	 = pgm_read_byte(&font_adress[2]);  //get first defined character
	last_code	 = pgm_read_byte(&font_adress[3]);  //get last defined character
	width		 = pgm_read_byte(&font_adress[4]);  //width in pixel of one char
	page_height  = pgm_read_byte(&font_adress[6]);  //page count per char
	bytes_p_char = pgm_read_byte(&font_adress[7]);  //bytes per char
  
  if(page_height + page > 8) //stay inside display area
		page_height = 8 - page;
  
	
	//The string is displayed character after character. If the font has more then one page, 
	//the top page is printed first, then the next page and so on
	for(y = 0; y < page_height; y++)
	{
		position(column, page+y); //set startpositon and page
		column_cnt = column; //store column for display last column check
		string = str;             //temporary pointer to the beginning of the string to print

		while(*string != 0)
		{	
			if((byte)*string < start_code || (byte)*string > last_code) //make sure data is valid
				string++;
			else
			{							
				//calculate positon of ascii character in font array
				//bytes for header + (ascii - startcode) * bytes per char)
				pos_array = 8 + (unsigned int)(*string++ - start_code) * bytes_p_char;
				pos_array += y*width; //get the dot pattern for the part of the char to print
        
				if(column_cnt + width > 128) //stay inside display area
					width_max = 128-column_cnt;
				else
					width_max = width;
				for(x=0; x < width_max; x++) //print the whole string
				{
					iic_put_byte(pgm_read_byte(&font_adress[pos_array+x]),0);
					//iic_put_byte(pgm_read_byte(&font_adress[pos_array+x]),0); //double width font (bold)
				}
			}
		}
	}
}

/*----------------------------
Func: rectangle
Desc: shows a pattern filled rectangle on the display
Vars: start and end column (0..127/131) and page(0..3/7), bit pattern
------------------------------*/
void oled_SSD1309::rectangle(byte start_column, byte start_page, byte end_column, byte end_page, byte pattern)  
{
	byte x, y;
  
  if( end_column > 128) //stay inside display area
		end_column = 128;

	if(end_page > 7)
		end_page = 7;

    
  for(y=start_page; y<=end_page; y++)
	{
		position(start_column, y);	
         
		
		for(x=start_column; x<=end_column; x++)
			iic_put_byte(pattern,0);

	}
}

/*----------------------------
Func: picture
Desc: shows a BLH-picture on the display (see BitMapEdit EA LCD-Tools (http://www.lcd-module.de/support.html))
Vars: column (0..127/131) and page(0..3/7), program memory adress of data
------------------------------*/
void oled_SSD1309::picture(byte column, byte page, const byte *pic_adress)  
{
	byte c,p;
	unsigned int byte_cnt = 2;
	byte width, page_cnt;
		
	width = pgm_read_byte(&pic_adress[0]);
	page_cnt = (pgm_read_byte(&pic_adress[1]) + 7) / 8; //height in pages, add 7 and divide by 8 for getting the used pages (byte boundaries)
	     
  if(width + column > 128 ) //stay inside display area
		width = 128 - column;
  
  if(page_cnt + page > 8)
		page_cnt = 8 - page;

	
	for(p=0; p<page_cnt; p++)
	{
		position(column, page + p);
		
		for(c=0; c<width; c++)
			iic_put_byte(pgm_read_byte(&pic_adress[byte_cnt++]),0);

	}
}


void oled_SSD1309::readDisplay()
{
  int i;
  position(10,1);

  Serial.print(128,DEC);
  Serial.print(64,DEC);
  
	Wire.beginTransmission(iic_address);
	Wire.write(0x40);
	Wire.endTransmission();
  
  //for(i=0; i<128*8; i+=32)
  {
    Wire.requestFrom(iic_address, 32, 1);    // request 32 bytes from slave device #2
    while(Wire.available())    // slave may send less than requested
    {
      char c = Wire.read();    // receive a byte as character
      Serial.print(c);         // print the character
    }
  }
}

//----------------------------------------------------private Functions----------------------------------------------------
//normally you don't need those functions in your sketch

/*----------------------------
Func: position
Desc: sets write pointer in OLED-Display
Vars: column (0..127/131), page(0..3/7)
------------------------------*/
void oled_SSD1309::position(byte column, byte page)  
{
	//if(top_view)
	//	column += 4;
		
	command(0x10 + (column>>4)); 	//MSB adress column
	command(0x00 + (column&0x0F));	//LSB adress column
	command(0xB0 + (page&0x0F)); 	//adress page	
}

/*----------------------------
Func: command
Desc: Sends a command to the OLED-Display
Vars: data
------------------------------*/
void oled_SSD1309::command(byte dat) 
{
	iic_put_byte(dat, 1);
}

/*----------------------------
Func: data
Desc: Sends data to the OLED-Display
Vars: data
------------------------------*/
void oled_SSD1309::data(byte dat) 
{
	 iic_put_byte(dat, 0);
}


/*----------------------------
Func: spi_put_byte
Desc: Sends one Byte
Vars: data
------------------------------*/
void oled_SSD1309::iic_put_byte(byte dat, byte command) 
{
	iic_out(dat, command);
}

/*----------------------------
Func: spi_put
Desc: Sends bytes
Vars: ptr to data and len
------------------------------*/
void oled_SSD1309::iic_put(byte *dat, int len, byte command) 
{
	byte controlbyte = (command^1) << 6;
	Wire.beginTransmission(iic_address);
	Wire.write(controlbyte);
	Wire.write(dat, len);
	Wire.endTransmission();

}

/*----------------------------
Func: spi_out
Desc: Sends one Byte
Vars: data
------------------------------*/
void oled_SSD1309::iic_out(byte dat, byte command) 
{
	byte controlbyte = ((command^1) << 6) | 0x80;
	Wire.beginTransmission(iic_address);
	Wire.write(controlbyte);
	Wire.write(dat);
	Wire.endTransmission();
}

